'''Crude audiobook support for MPD (Music Player Daemon).'''

import argparse
import contextlib
import logging
import os
import typing

import musicpd

from mpdab.commands import do_add, do_continue, do_list, do_next, do_watch
from mpdab.exceptions import MpdabException

__version__ = '0.1.0a2'

_LOG = logging.getLogger(__name__)


@contextlib.contextmanager
def get_client_from_args(
        args: argparse.Namespace,
) -> typing.Iterator[musicpd.MPDClient]:
    pwd, host = None, args.host
    if '@' in host:
        pwd, host = host.split('@', 1)

    client = musicpd.MPDClient()
    client.pwd = pwd
    client.host = host
    client.port = args.port

    client.connect()
    if client.pwd:
        client.password(client.pwd)
    yield client
    client.disconnect()


def main():
    parser = argparse.ArgumentParser(
        prog='mpdab',
        description='Crude audiobook support for Music Player Daemon',
    )
    parser.add_argument(
        '--version',
        action='version',
        version=f'%(prog)s {__version__}',
    )
    parser.add_argument(
        '--debug',
        action='store_true',
    )
    parser.add_argument(
        '--host',
        help='the MPD server to connect to. This can be a hostname, IPv4/IPv6 '
             'address, an absolute path (i.e. local socket) or a name '
             'starting with @ (i.e. an abstract socket, Linux only). '
             'Default: %(default)s',
        default=os.getenv('MPD_HOST', 'localhost'),
    )
    parser.add_argument(
        '-p',
        '--port',
        type=int,
        default=os.getenv('MPD_PORT', 6600),
        help='the TCP port of the MPD server to connect to.'
             'If not given, the value of the environment variable MPD_PORT is '
             'used. Default: %(default)s',
    )
    subparsers = parser.add_subparsers()

    add_parser = subparsers.add_parser('add')
    add_parser.add_argument(
        'name',
    )
    add_parser.add_argument(
        'filenames',
        nargs='+',
        default=['-'],
    )
    add_parser.set_defaults(func=do_add)

    list_parser = subparsers.add_parser('list', aliases=('ls',))
    list_parser.set_defaults(func=do_list)

    continue_parser = subparsers.add_parser('continue')
    continue_parser.add_argument(
        'name',
    )
    continue_parser.set_defaults(func=do_continue)

    next_parser = subparsers.add_parser('next')
    next_parser.set_defaults(func=do_next)

    watch_parser = subparsers.add_parser('watch')
    watch_parser.set_defaults(func=do_watch)

    args = parser.parse_args()

    logging.basicConfig(level=logging.DEBUG if args.debug else logging.INFO)

    func = getattr(args, 'func', lambda *a: parser.print_usage())
    try:
        with get_client_from_args(args) as client:
            func(args, client)
    except MpdabException as e:
        raise SystemExit(str(e))
    except KeyboardInterrupt:
        raise SystemExit('Interrupted by user, exiting...')


if __name__ == '__main__':
    main()
