import datetime
import logging
import typing

import musicpd
from mpdab.exceptions import MpdabException
from mpdab.item import Item, get_item_from_sticker_map, parse_stickers

_LOG = logging.getLogger(__name__)


def get_item(client: musicpd.MPDClient, song_uri: str) -> Item:
    return get_item_from_sticker_map(
        song_uri, parse_stickers(client.sticker_list('song', song_uri)))


def get_items(
        client: musicpd.MPDClient,
        song_uris: typing.List[str],
) -> typing.List[Item]:
    client.command_list_ok_begin()
    for song_uri in song_uris:
        client.sticker_list('song', song_uri)
    return [
        get_item_from_sticker_map(song_uri, parse_stickers(stickers))
        for song_uri, stickers in zip(song_uris, client.command_list_end())
    ]


def get_audiobook_items(client: musicpd.MPDClient,
                        audiobook_name: str) -> typing.List[Item]:
    song_uris = [
        i['file'] for i in client.sticker_find(
            'song', '', 'audiobook', '=', audiobook_name)
    ]
    if not song_uris:
        raise MpdabException(
            f'No such audiobook "{audiobook_name}".')

    return sorted(get_items(client, song_uris))


def get_all_audiobooks(client: musicpd.MPDClient) -> typing.List[str]:
    audiobooks = []
    seen: typing.Set[str] = set()
    for item in client.sticker_find('song', '', 'audiobook'):
        audiobook = parse_stickers([item['sticker']])['audiobook']
        if audiobook in seen:
            continue
        audiobooks.append(audiobook)
        seen.add(audiobook)
    return audiobooks


def store_playback_bookmark(
        client: musicpd.MPDClient,
        item: Item,
        position: str,
        now: typing.Optional[datetime.datetime] = None,
) -> None:
    now = now or datetime.datetime.now()
    _LOG.debug('Storing playback bookmark for item: %s at position: %s on %s.',
               item, position, now)
    client.command_list_ok_begin()
    client.sticker_set(
        'song', item.uri, 'last_position', position,
    )
    client.sticker_set(
        'song', item.uri, 'last_played', now.isoformat(),
    )
    client.command_list_end()


def enqueue(client: musicpd.MPDClient, items: typing.List[Item],
            resume: bool = True, autoplay: bool = True) -> None:
    # If resumption is enabled, try to find the last playing position.
    last_played, last_played_idx = None, -1
    if resume:
        # Sort by last played date in descending order and take the first one
        # (latest).
        bookmarks = sorted(
            (i for i in items if i.last_played),
            key=lambda i: i.last_played,
            reverse=True,
        )
        if bookmarks:
            last_played = bookmarks[0]
            last_played_idx = items.index(last_played)

    client.command_list_ok_begin()

    # Stop current playback.
    client.stop()

    # Clear current queue.
    client.clear()

    # Turn shuffle off.
    client.random(0)

    # Turn song consumation off.
    client.consume(0)

    # Turn song repetition off.
    client.repeat(0)

    # Turn single song off.
    client.single(0)

    # Turn crossfade off.
    # TODO: Is this needed?
    # client.crossfade(0)

    # Enqueue all the items.
    for item in items:
        client.add(item.uri)

    # Seek to the last playback position (if any).
    if resume and last_played and last_played_idx >= 0:
        pos = max((0.0, (last_played.last_position or 0.0) - 5.0))
        _LOG.debug('Seeking last played item: %s to position: %.2f.',
                   last_played, pos)
        client.seek(last_played_idx, pos)
        # `seek()` starts playback, pause it.
        client.pause()

    # Resume playback.
    if autoplay:
        client.play()

    client.command_list_end()
